@@ -1,4 +1,4 @@
-# last modified 14 April 2009 by J. Fox + slight changes 12 Aug 04 by Ph. Grosjean
+# last modified 22 August 2009 by J. Fox + slight changes 12 Aug 04 by Ph. Grosjean
 
 # utility functions
 
@@ -509,6 +509,24 @@ print.numSummary <- function(x, ...){
         )
     invisible(x)
     }
+	
+stepwise <- function(mod, 
+			direction=c("backward/forward", "forward/backward", "backward", "forward"), 
+			criterion=c("BIC", "AIC"), ...){
+		if (!require(MASS)) stop("MASS package not available")
+		criterion <- match.arg(criterion)
+		cat("\nDirection: ", direction)
+		cat("\nCriterion: ", criterion, "\n\n")
+		k <- if (criterion == "BIC") log(length(na.omit(residuals(mod)))) else 2
+		rhs <- paste(c("~", deparse(formula(mod)[[3]])), collapse="")
+		rhs <- gsub(" ", "", rhs)
+		if (direction == "forward" || direction == "forward/backward")
+			mod <- update(mod, . ~ 1)
+		if (direction == "backward/forward" || direction == "forward/backward") direction <- "both"
+		lower <- ~ 1
+		upper <- eval(parse(text=rhs))   
+		stepAIC(mod, scope=list(lower=lower, upper=upper), direction=direction, k=k, ...)
+	}
 
     # wrapper function for histograms
 
@@ -533,74 +551,74 @@ Hist <- function(x, scale=c("frequency", "percent", "density"), xlab=deparse(sub
 	invisible(NULL)
 }
 
-	plotMeans <- function(response, factor1, factor2, error.bars = c("se", "sd", "conf.int", "none"),
-		level=0.95, xlab=deparse(substitute(factor1)), ylab=paste("mean of", deparse(substitute(response))),
-		legend.lab=deparse(substitute(factor2)), main="Plot of Means",
-		pch=1:n.levs.2, lty=1:n.levs.2, col=palette()){
-		if (!is.numeric(response)) stop(gettextRcmdr("Argument response must be numeric."))
-		xlab # force evaluation
-		ylab
-		legend.lab
-		error.bars <- match.arg(error.bars)
-		if (missing(factor2)){
-			if (!is.factor(factor1)) stop(gettextRcmdr("Argument factor1 must be a factor."))
-			valid <- complete.cases(factor1, response)
-			factor1 <- factor1[valid]
-			response <- response[valid]
-			means <- tapply(response, factor1, mean)
-			sds <- tapply(response, factor1, sd)
-			ns <- tapply(response, factor1, length)
-			if (error.bars == "se") sds <- sds/sqrt(ns)
-			if (error.bars == "conf.int") sds <- qt((1 - level)/2, df=ns - 1, lower.tail=FALSE) * sds/sqrt(ns)
-			sds[is.na(sds)] <- 0
-			yrange <-  if (error.bars != "none") c( min(means - sds, na.rm=TRUE), max(means + sds, na.rm=TRUE)) else range(means, na.rm=TRUE)
-			levs <- levels(factor1)
-			n.levs <- length(levs)
-			plot(c(1, n.levs), yrange, type="n", xlab=xlab, ylab=ylab, axes=FALSE, main=main)
-			points(1:n.levs, means, type="b", pch=16, cex=2)
-			box()
-			axis(2)
-			axis(1, at=1:n.levs, labels=levs)
-			if (error.bars != "none") arrows(1:n.levs, means - sds, 1:n.levs, means + sds,
-					angle=90, lty=2, code=3, length=0.125)
-		}
-		else {
-			if (!(is.factor(factor1) | is.factor(factor2))) stop(gettextRcmdr("Arguments factor1 and factor2 must be factors."))
-			valid <- complete.cases(factor1, factor2, response)
-			factor1 <- factor1[valid]
-			factor2 <- factor2[valid]
-			response <- response[valid]
-			means <- tapply(response, list(factor1, factor2), mean)
-			sds <- tapply(response, list(factor1, factor2), sd)
-			ns <- tapply(response, list(factor1, factor2), length)
-			if (error.bars == "se") sds <- sds/sqrt(ns)
-			if (error.bars == "conf.int") sds <- qt((1 - level)/2, df=ns - 1, lower.tail=FALSE) * sds/sqrt(ns)
-			sds[is.na(sds)] <- 0
-			yrange <-  if (error.bars != "none") c( min(means - sds, na.rm=TRUE), max(means + sds, na.rm=TRUE)) else range(means, na.rm=TRUE)
-			levs.1 <- levels(factor1)
-			levs.2 <- levels(factor2)
-			n.levs.1 <- length(levs.1)
-			n.levs.2 <- length(levs.2)
-			if (length(pch) == 1) pch <- rep(pch, n.levs.2)
-			if (length(col) == 1) col <- rep(col, n.levs.2)
-			if (length(lty) == 1) lty <- rep(lty, n.levs.2)
-			if (n.levs.2 > length(col)) stop(sprintf(gettextRcmdr("Number of groups for factor2, %d, exceeds number of distinct colours, %d."), n.levs.2, length(col)))		
-			plot(c(1, n.levs.1 * 1.4), yrange, type="n", xlab=xlab, ylab=ylab, axes=FALSE, main=main)
-			box()
-			axis(2)
-			axis(1, at=1:n.levs.1, labels=levs.1)
-			for (i in 1:n.levs.2){
-				points(1:n.levs.1, means[, i], type="b", pch=pch[i], cex=2, col=col[i], lty=lty[i])
-				if (error.bars != "none") arrows(1:n.levs.1, means[, i] - sds[, i],
-						1:n.levs.1, means[, i] + sds[, i], angle=90, code=3, col=col[i], lty=lty[i], length=0.125)
-			}
-			x.posn <- n.levs.1 * 1.1
-			y.posn <- sum(c(0.1, 0.9) * par("usr")[c(3,4)])
-			text(x.posn, y.posn, legend.lab, adj=c(0, -.5))
-			legend(x.posn, y.posn, levs.2, pch=pch, col=col, lty=lty)
-		}
-		invisible(NULL)
+plotMeans <- function(response, factor1, factor2, error.bars = c("se", "sd", "conf.int", "none"),
+	level=0.95, xlab=deparse(substitute(factor1)), ylab=paste("mean of", deparse(substitute(response))),
+	legend.lab=deparse(substitute(factor2)), main="Plot of Means",
+	pch=1:n.levs.2, lty=1:n.levs.2, col=palette()){
+	if (!is.numeric(response)) stop(gettextRcmdr("Argument response must be numeric."))
+	xlab # force evaluation
+	ylab
+	legend.lab
+	error.bars <- match.arg(error.bars)
+	if (missing(factor2)){
+		if (!is.factor(factor1)) stop(gettextRcmdr("Argument factor1 must be a factor."))
+		valid <- complete.cases(factor1, response)
+		factor1 <- factor1[valid]
+		response <- response[valid]
+		means <- tapply(response, factor1, mean)
+		sds <- tapply(response, factor1, sd)
+		ns <- tapply(response, factor1, length)
+		if (error.bars == "se") sds <- sds/sqrt(ns)
+		if (error.bars == "conf.int") sds <- qt((1 - level)/2, df=ns - 1, lower.tail=FALSE) * sds/sqrt(ns)
+		sds[is.na(sds)] <- 0
+		yrange <-  if (error.bars != "none") c( min(means - sds, na.rm=TRUE), max(means + sds, na.rm=TRUE)) else range(means, na.rm=TRUE)
+		levs <- levels(factor1)
+		n.levs <- length(levs)
+		plot(c(1, n.levs), yrange, type="n", xlab=xlab, ylab=ylab, axes=FALSE, main=main)
+		points(1:n.levs, means, type="b", pch=16, cex=2)
+		box()
+		axis(2)
+		axis(1, at=1:n.levs, labels=levs)
+		if (error.bars != "none") arrows(1:n.levs, means - sds, 1:n.levs, means + sds,
+				angle=90, lty=2, code=3, length=0.125)
 	}
+	else {
+		if (!(is.factor(factor1) | is.factor(factor2))) stop(gettextRcmdr("Arguments factor1 and factor2 must be factors."))
+		valid <- complete.cases(factor1, factor2, response)
+		factor1 <- factor1[valid]
+		factor2 <- factor2[valid]
+		response <- response[valid]
+		means <- tapply(response, list(factor1, factor2), mean)
+		sds <- tapply(response, list(factor1, factor2), sd)
+		ns <- tapply(response, list(factor1, factor2), length)
+		if (error.bars == "se") sds <- sds/sqrt(ns)
+		if (error.bars == "conf.int") sds <- qt((1 - level)/2, df=ns - 1, lower.tail=FALSE) * sds/sqrt(ns)
+		sds[is.na(sds)] <- 0
+		yrange <-  if (error.bars != "none") c( min(means - sds, na.rm=TRUE), max(means + sds, na.rm=TRUE)) else range(means, na.rm=TRUE)
+		levs.1 <- levels(factor1)
+		levs.2 <- levels(factor2)
+		n.levs.1 <- length(levs.1)
+		n.levs.2 <- length(levs.2)
+		if (length(pch) == 1) pch <- rep(pch, n.levs.2)
+		if (length(col) == 1) col <- rep(col, n.levs.2)
+		if (length(lty) == 1) lty <- rep(lty, n.levs.2)
+		if (n.levs.2 > length(col)) stop(sprintf(gettextRcmdr("Number of groups for factor2, %d, exceeds number of distinct colours, %d."), n.levs.2, length(col)))		
+		plot(c(1, n.levs.1 * 1.4), yrange, type="n", xlab=xlab, ylab=ylab, axes=FALSE, main=main)
+		box()
+		axis(2)
+		axis(1, at=1:n.levs.1, labels=levs.1)
+		for (i in 1:n.levs.2){
+			points(1:n.levs.1, means[, i], type="b", pch=pch[i], cex=2, col=col[i], lty=lty[i])
+			if (error.bars != "none") arrows(1:n.levs.1, means[, i] - sds[, i],
+					1:n.levs.1, means[, i] + sds[, i], angle=90, code=3, col=col[i], lty=lty[i], length=0.125)
+		}
+		x.posn <- n.levs.1 * 1.1
+		y.posn <- sum(c(0.1, 0.9) * par("usr")[c(3,4)])
+		text(x.posn, y.posn, legend.lab, adj=c(0, -.5))
+		legend(x.posn, y.posn, levs.2, pch=pch, col=col, lty=lty)
+	}
+	invisible(NULL)
+}
 
 bin.var <- function (x, bins=4, method=c("intervals", "proportions", "natural"), labels=FALSE){
     method <- match.arg(method)
@@ -619,6 +637,30 @@ bin.var <- function (x, bins=4, method=c("intervals", "proportions", "natural"),
     as.factor(x)
     }
 
+# the following function is adapted from a suggestion by Robert Muenchen
+
+rcorr.adjust <- function(x, type=c("pearson", "spearman")){
+	require("Hmisc")
+	type <- match.arg(type)
+	x <- as.matrix(na.omit(x))
+	R <- rcorr(x, type=type)
+	P <- R$P
+	p <- P[lower.tri(P)]
+	adj.p <- p.adjust(p, method="holm")
+	P[lower.tri(P)] <- adj.p
+	P[upper.tri(P)] <- 0
+	P <- P + t(P)
+	P <- ifelse(P < 1e-04, 0, P)
+	P <- format(round(P, 4))
+	P[grep("NA", P)] <- ""
+	print(R)
+	cat("\n Adjusted p-values (Holm's method)\n")
+	print(P, quote = FALSE)
+	R$adj.P <- P
+	return(invisible(R))
+}
+
+
 # 3D scatterplots and point identification via rgl
 
 ellipsoid <- function(center=c(0, 0, 0), radius=1, shape=diag(3), n=30){
@@ -1251,8 +1293,10 @@ dialogSuffix <- defmacro(window=top, onOK=onOK, rows=1, columns=1, focus=top,
 		if (selectmode == "multiple") selectmode <- getRcmdr("multiple.select.mode")
 		if (length(variableList) == 1 && is.null(initialSelection)) initialSelection <- 0
 		frame <- tkframe(parentWindow)
+		minmax <- getRcmdr("variable.list.width")
 		listbox <- tklistbox(frame, height=min(listHeight, length(variableList)),
-			selectmode=selectmode, background=bg, exportselection=export, width=max(20, nchar(variableList)))
+			selectmode=selectmode, background=bg, exportselection=export, 
+			width=min(max(minmax[1], nchar(variableList)), minmax[2]))
 		scrollbar <- ttkscrollbar(frame, command=function(...) tkyview(listbox, ...))
 		tkconfigure(listbox, yscrollcommand=function(...) tkset(scrollbar, ...))
 		for (var in variableList) tkinsert(listbox, "end", var)
@@ -1466,169 +1510,358 @@ groupsLabel <- defmacro(frame=top, groupsBox=groupsBox, columnspan=1,
         tkbind(groupsBox$listbox, "<ButtonRelease-1>", onSelect)
         })
 
-modelFormula <- defmacro(frame=top, hasLhs=TRUE, expr={
-    checkAddOperator <- function(rhs){
-        rhs.chars <- rev(strsplit(rhs, "")[[1]])
-        if (length(rhs.chars) < 1) return(FALSE)
-        check.char <- if ((rhs.chars[1] != " ") || (length(rhs.chars) == 1))
-                rhs.chars[1] else rhs.chars[2]
-        !is.element(check.char, c("+", "*", ":", "/", "-", "^", "(", "%"))
-        }
-    .variables <- Variables()
-    word <- paste("\\[", gettextRcmdr("factor"), "\\]", sep="")
-    variables <- paste(.variables,
-        ifelse(is.element(.variables, Factors()), paste("[", gettextRcmdr("factor"), "]", sep=""), ""))
-    xBox <- variableListBox(frame, variables, title=gettextRcmdr("Variables (double-click to formula)"))
-    onDoubleClick <- if (!hasLhs){
-        function(){
-            var <- getSelection(xBox)
-            if (length(grep(word, var)) == 1) var <- sub(word, "",  var)
-            tkfocus(rhsEntry)
-            rhs <- tclvalue(rhsVariable)
-            rhs.chars <- rev(strsplit(rhs, "")[[1]])
-            check.char <- if (length(rhs.chars) > 0){
-                if ((rhs.chars[1] != " ") || (length(rhs.chars) == 1))
-                    rhs.chars[1] else rhs.chars[2]
-                }
-                else ""
-            tclvalue(rhsVariable) <- if (rhs == "" ||
-                is.element(check.char, c("+", "*", ":", "/", "-", "^", "(", "%")))
-                    paste(rhs, var, sep="")
-                else paste(rhs, "+", var)
-            tkicursor(rhsEntry, "end")
-            tkxview.moveto(rhsEntry, "1")
-            }
-        }
-    else{
-        function(){
-            var <- getSelection(xBox)
-            if (length(grep(word, var)) == 1) var <- sub(word, "",  var)
-            lhs <- tclvalue(lhsVariable)
-            if (lhs == "") tclvalue(lhsVariable) <- var
-            else {
-                tkfocus(rhsEntry)
-                rhs <- tclvalue(rhsVariable)
-                rhs.chars <- rev(strsplit(rhs, "")[[1]])
-                check.char <- if (length(rhs.chars) > 0){
-                    if ((rhs.chars[1] != " ") || (length(rhs.chars) == 1))
-                        rhs.chars[1] else rhs.chars[2]
-                    }
-                    else ""
-                tclvalue(rhsVariable) <- if (rhs == "" ||
-                    is.element(check.char, c("+", "*", ":", "/", "-", "^", "(", "%")))
-                        paste(rhs, var, sep="")
-                    else paste(rhs, "+", var)
-                }
-            tkicursor(rhsEntry, "end")
-            tkxview.moveto(rhsEntry, "1")
-            }
-        }
-    tkbind(xBox$listbox, "<Double-ButtonPress-1>", onDoubleClick)
-    onPlus <- function(){
-        rhs <- tclvalue(rhsVariable)
-        if (!checkAddOperator(rhs)) return()
-        tclvalue(rhsVariable) <- paste(rhs, "+ ")
-        tkicursor(rhsEntry, "end")
-        tkxview.moveto(rhsEntry, "1")
-        }
-    onTimes <- function(){
-        rhs <- tclvalue(rhsVariable)
-        if (!checkAddOperator(rhs)) return()
-        tclvalue(rhsVariable) <- paste(rhs, "*", sep="")
-        tkicursor(rhsEntry, "end")
-        tkxview.moveto(rhsEntry, "1")
-        }
-    onColon <- function(){
-        rhs <- tclvalue(rhsVariable)
-        if (!checkAddOperator(rhs)) return()
-        tclvalue(rhsVariable) <- paste(rhs, ":", sep="")
-        tkicursor(rhsEntry, "end")
-        tkxview.moveto(rhsEntry, "1")
-        }
-    onSlash <- function(){
-        rhs <- tclvalue(rhsVariable)
-        if (!checkAddOperator(rhs)) return()
-        tclvalue(rhsVariable) <- paste(rhs, "/",  sep="")
-        tkicursor(rhsEntry, "end")
-        tkxview.moveto(rhsEntry, "1")
-        }
-    onIn <- function(){
-        rhs <- tclvalue(rhsVariable)
-        if (!checkAddOperator(rhs)) return()
-        tclvalue(rhsVariable) <- paste(rhs, "%in% ")
-        tkicursor(rhsEntry, "end")
-        tkxview.moveto(rhsEntry, "1")
-        }
-    onMinus <- function(){
-        rhs <- tclvalue(rhsVariable)
-        if (!checkAddOperator(rhs)) return()
-        tclvalue(rhsVariable) <- paste(rhs, "- ")
-        tkicursor(rhsEntry, "end")
-        tkxview.moveto(rhsEntry, "1")
-        }
-    onPower <- function(){
-        rhs <- tclvalue(rhsVariable)
-        if (!checkAddOperator(rhs)) return()
-        tclvalue(rhsVariable) <- paste(rhs, "^", sep="")
-        tkicursor(rhsEntry, "end")
-        tkxview.moveto(rhsEntry, "1")
-        }
-    onLeftParen <- function(){
-        tkfocus(rhsEntry)
-        rhs <- tclvalue(rhsVariable)
-        tclvalue(rhsVariable) <- paste(rhs, "(", sep="")
-        tkicursor(rhsEntry, "end")
-        tkxview.moveto(rhsEntry, "1")
-        }
-    onRightParen <- function(){
-        rhs <- tclvalue(rhsVariable)
-        if (!checkAddOperator(rhs)) return()
-        tclvalue(rhsVariable) <- paste(rhs, ")", sep="")
-        tkicursor(rhsEntry, "end")
-        tkxview.moveto(rhsEntry, "1")
-        }
-    outerOperatorsFrame <- tkframe(frame)
-    operatorsFrame <- tkframe(outerOperatorsFrame)
-    plusButton <- buttonRcmdr(operatorsFrame, text="+", width="3", command=onPlus)
-    timesButton <- buttonRcmdr(operatorsFrame, text="*", width="3", command=onTimes)
-    colonButton <- buttonRcmdr(operatorsFrame, text=":", width="3", command=onColon)
-    slashButton <- buttonRcmdr(operatorsFrame, text="/", width="3", command=onSlash)
-    inButton <- buttonRcmdr(operatorsFrame, text="%in%", width="5", command=onIn)
-    minusButton <- buttonRcmdr(operatorsFrame, text="-", width="3", command=onMinus)
-    powerButton <- buttonRcmdr(operatorsFrame, text="^", width="3", command=onPower)
-    leftParenButton <- buttonRcmdr(operatorsFrame, text="(", width="3", command=onLeftParen)
-    rightParenButton <- buttonRcmdr(operatorsFrame, text=")", width="3", command=onRightParen)
+#modelFormula <- defmacro(frame=top, hasLhs=TRUE, expr={
+#    checkAddOperator <- function(rhs){
+#        rhs.chars <- rev(strsplit(rhs, "")[[1]])
+#        if (length(rhs.chars) < 1) return(FALSE)
+#        check.char <- if ((rhs.chars[1] != " ") || (length(rhs.chars) == 1))
+#                rhs.chars[1] else rhs.chars[2]
+#        !is.element(check.char, c("+", "*", ":", "/", "-", "^", "(", "%"))
+#        }
+#    .variables <- Variables()
+#    word <- paste("\\[", gettextRcmdr("factor"), "\\]", sep="")
+#    variables <- paste(.variables,
+#        ifelse(is.element(.variables, Factors()), paste("[", gettextRcmdr("factor"), "]", sep=""), ""))
+#    xBox <- variableListBox(frame, variables, title=gettextRcmdr("Variables (double-click to formula)"))
+#    onDoubleClick <- if (!hasLhs){
+#        function(){
+#            var <- getSelection(xBox)
+#            if (length(grep(word, var)) == 1) var <- sub(word, "",  var)
+#            tkfocus(rhsEntry)
+#            rhs <- tclvalue(rhsVariable)
+#            rhs.chars <- rev(strsplit(rhs, "")[[1]])
+#            check.char <- if (length(rhs.chars) > 0){
+#                if ((rhs.chars[1] != " ") || (length(rhs.chars) == 1))
+#                    rhs.chars[1] else rhs.chars[2]
+#                }
+#                else ""
+#            tclvalue(rhsVariable) <- if (rhs == "" ||
+#                is.element(check.char, c("+", "*", ":", "/", "-", "^", "(", "%")))
+#                    paste(rhs, var, sep="")
+#                else paste(rhs, "+", var)
+#            tkicursor(rhsEntry, "end")
+#            tkxview.moveto(rhsEntry, "1")
+#            }
+#        }
+#    else{
+#        function(){
+#            var <- getSelection(xBox)
+#            if (length(grep(word, var)) == 1) var <- sub(word, "",  var)
+#            lhs <- tclvalue(lhsVariable)
+#            if (lhs == "") tclvalue(lhsVariable) <- var
+#            else {
+#                tkfocus(rhsEntry)
+#                rhs <- tclvalue(rhsVariable)
+#                rhs.chars <- rev(strsplit(rhs, "")[[1]])
+#                check.char <- if (length(rhs.chars) > 0){
+#                    if ((rhs.chars[1] != " ") || (length(rhs.chars) == 1))
+#                        rhs.chars[1] else rhs.chars[2]
+#                    }
+#                    else ""
+#                tclvalue(rhsVariable) <- if (rhs == "" ||
+#                    is.element(check.char, c("+", "*", ":", "/", "-", "^", "(", "%")))
+#                        paste(rhs, var, sep="")
+#                    else paste(rhs, "+", var)
+#                }
+#            tkicursor(rhsEntry, "end")
+#            tkxview.moveto(rhsEntry, "1")
+#            }
+#        }
+#    tkbind(xBox$listbox, "<Double-ButtonPress-1>", onDoubleClick)
+#    onPlus <- function(){
+#        rhs <- tclvalue(rhsVariable)
+#        if (!checkAddOperator(rhs)) return()
+#        tclvalue(rhsVariable) <- paste(rhs, "+ ")
+#        tkicursor(rhsEntry, "end")
+#        tkxview.moveto(rhsEntry, "1")
+#        }
+#    onTimes <- function(){
+#        rhs <- tclvalue(rhsVariable)
+#        if (!checkAddOperator(rhs)) return()
+#        tclvalue(rhsVariable) <- paste(rhs, "*", sep="")
+#        tkicursor(rhsEntry, "end")
+#        tkxview.moveto(rhsEntry, "1")
+#        }
+#    onColon <- function(){
+#        rhs <- tclvalue(rhsVariable)
+#        if (!checkAddOperator(rhs)) return()
+#        tclvalue(rhsVariable) <- paste(rhs, ":", sep="")
+#        tkicursor(rhsEntry, "end")
+#        tkxview.moveto(rhsEntry, "1")
+#        }
+#    onSlash <- function(){
+#        rhs <- tclvalue(rhsVariable)
+#        if (!checkAddOperator(rhs)) return()
+#        tclvalue(rhsVariable) <- paste(rhs, "/",  sep="")
+#        tkicursor(rhsEntry, "end")
+#        tkxview.moveto(rhsEntry, "1")
+#        }
+#    onIn <- function(){
+#        rhs <- tclvalue(rhsVariable)
+#        if (!checkAddOperator(rhs)) return()
+#        tclvalue(rhsVariable) <- paste(rhs, "%in% ")
+#        tkicursor(rhsEntry, "end")
+#        tkxview.moveto(rhsEntry, "1")
+#        }
+#    onMinus <- function(){
+#        rhs <- tclvalue(rhsVariable)
+#        if (!checkAddOperator(rhs)) return()
+#        tclvalue(rhsVariable) <- paste(rhs, "- ")
+#        tkicursor(rhsEntry, "end")
+#        tkxview.moveto(rhsEntry, "1")
+#        }
+#    onPower <- function(){
+#        rhs <- tclvalue(rhsVariable)
+#        if (!checkAddOperator(rhs)) return()
+#        tclvalue(rhsVariable) <- paste(rhs, "^", sep="")
+#        tkicursor(rhsEntry, "end")
+#        tkxview.moveto(rhsEntry, "1")
+#        }
+#    onLeftParen <- function(){
+#        tkfocus(rhsEntry)
+#        rhs <- tclvalue(rhsVariable)
+#        tclvalue(rhsVariable) <- paste(rhs, "(", sep="")
+#        tkicursor(rhsEntry, "end")
+#        tkxview.moveto(rhsEntry, "1")
+#        }
+#    onRightParen <- function(){
+#        rhs <- tclvalue(rhsVariable)
+#        if (!checkAddOperator(rhs)) return()
+#        tclvalue(rhsVariable) <- paste(rhs, ")", sep="")
+#        tkicursor(rhsEntry, "end")
+#        tkxview.moveto(rhsEntry, "1")
+#        }
+#    outerOperatorsFrame <- tkframe(frame)
+#    operatorsFrame <- tkframe(outerOperatorsFrame)
+#    plusButton <- buttonRcmdr(operatorsFrame, text="+", width="3", command=onPlus)
+#    timesButton <- buttonRcmdr(operatorsFrame, text="*", width="3", command=onTimes)
+#    colonButton <- buttonRcmdr(operatorsFrame, text=":", width="3", command=onColon)
+#    slashButton <- buttonRcmdr(operatorsFrame, text="/", width="3", command=onSlash)
+#    inButton <- buttonRcmdr(operatorsFrame, text="%in%", width="5", command=onIn)
+#    minusButton <- buttonRcmdr(operatorsFrame, text="-", width="3", command=onMinus)
+#    powerButton <- buttonRcmdr(operatorsFrame, text="^", width="3", command=onPower)
+#    leftParenButton <- buttonRcmdr(operatorsFrame, text="(", width="3", command=onLeftParen)
+#    rightParenButton <- buttonRcmdr(operatorsFrame, text=")", width="3", command=onRightParen)
+#
+#    tkgrid(plusButton, timesButton, colonButton, slashButton, inButton, minusButton,
+#        powerButton, leftParenButton, rightParenButton, sticky="w")
+#    formulaFrame <- tkframe(frame)
+#    if (hasLhs){
+#        tkgrid(labelRcmdr(outerOperatorsFrame, text=gettextRcmdr("Model Formula:     "), fg="blue"), operatorsFrame)
+#        lhsVariable <- if (currentModel) tclVar(currentFields$lhs) else tclVar("")
+#        rhsVariable <- if (currentModel) tclVar(currentFields$rhs) else tclVar("")
+#        rhsEntry <- ttkentry(formulaFrame, width="50", textvariable=rhsVariable)
+#        rhsXscroll <- ttkscrollbar(formulaFrame,
+#            orient="horizontal", command=function(...) tkxview(rhsEntry, ...))
+#        tkconfigure(rhsEntry, xscrollcommand=function(...) tkset(rhsXscroll, ...))
+#        lhsEntry <- ttkentry(formulaFrame, width="10", textvariable=lhsVariable)
+#        lhsScroll <- ttkscrollbar(formulaFrame,
+#            orient="horizontal", command=function(...) tkxview(lhsEntry, ...))
+#        tkconfigure(lhsEntry, xscrollcommand=function(...) tkset(lhsScroll, ...))
+#        tkgrid(lhsEntry, labelRcmdr(formulaFrame, text=" ~    "), rhsEntry, sticky="w")
+#        tkgrid(lhsScroll, labelRcmdr(formulaFrame, text=""), rhsXscroll, sticky="w")
+#        tkgrid.configure(lhsScroll, sticky="ew")
+#        }
+#    else{
+#        rhsVariable <- if (currentModel) tclVar(currentFields$rhs) else tclVar("")
+#        rhsEntry <- ttkentry(formulaFrame, width="50", textvariable=rhsVariable)
+#        rhsXscroll <- ttkscrollbar(formulaFrame,
+#            orient="horizontal", command=function(...) tkxview(rhs, ...))
+#        tkconfigure(rhsEntry, xscrollcommand=function(...) tkset(rhsXscroll, ...))
+#        tkgrid(labelRcmdr(formulaFrame, text="   ~ "), rhsEntry, sticky="w")
+#        tkgrid(labelRcmdr(formulaFrame, text=""), rhsXscroll, sticky="w")
+#        }
+#    tkgrid.configure(rhsXscroll, sticky="ew")
+#    })
 
-    tkgrid(plusButton, timesButton, colonButton, slashButton, inButton, minusButton,
-        powerButton, leftParenButton, rightParenButton, sticky="w")
-    formulaFrame <- tkframe(frame)
-    if (hasLhs){
-        tkgrid(labelRcmdr(outerOperatorsFrame, text=gettextRcmdr("Model Formula:     "), fg="blue"), operatorsFrame)
-        lhsVariable <- if (currentModel) tclVar(currentFields$lhs) else tclVar("")
-        rhsVariable <- if (currentModel) tclVar(currentFields$rhs) else tclVar("")
-        rhsEntry <- ttkentry(formulaFrame, width="50", textvariable=rhsVariable)
-        rhsXscroll <- ttkscrollbar(formulaFrame,
-            orient="horizontal", command=function(...) tkxview(rhsEntry, ...))
-        tkconfigure(rhsEntry, xscrollcommand=function(...) tkset(rhsXscroll, ...))
-        lhsEntry <- ttkentry(formulaFrame, width="10", textvariable=lhsVariable)
-        lhsScroll <- ttkscrollbar(formulaFrame,
-            orient="horizontal", command=function(...) tkxview(lhsEntry, ...))
-        tkconfigure(lhsEntry, xscrollcommand=function(...) tkset(lhsScroll, ...))
-        tkgrid(lhsEntry, labelRcmdr(formulaFrame, text=" ~    "), rhsEntry, sticky="w")
-        tkgrid(lhsScroll, labelRcmdr(formulaFrame, text=""), rhsXscroll, sticky="w")
-        tkgrid.configure(lhsScroll, sticky="ew")
-        }
-    else{
-        rhsVariable <- if (currentModel) tclVar(currentFields$rhs) else tclVar("")
-        rhsEntry <- ttkentry(formulaFrame, width="50", textvariable=rhsVariable)
-        rhsXscroll <- ttkscrollbar(formulaFrame,
-            orient="horizontal", command=function(...) tkxview(rhs, ...))
-        tkconfigure(rhsEntry, xscrollcommand=function(...) tkset(rhsXscroll, ...))
-        tkgrid(labelRcmdr(formulaFrame, text="   ~ "), rhsEntry, sticky="w")
-        tkgrid(labelRcmdr(formulaFrame, text=""), rhsXscroll, sticky="w")
-        }
-    tkgrid.configure(rhsXscroll, sticky="ew")
-    })
+modelFormula <- defmacro(frame=top, hasLhs=TRUE, expr={
+		checkAddOperator <- function(rhs){
+			rhs.chars <- rev(strsplit(rhs, "")[[1]])
+			if (length(rhs.chars) < 1) return(FALSE)
+			check.char <- if ((rhs.chars[1] != " ") || (length(rhs.chars) == 1))
+					rhs.chars[1] else rhs.chars[2]
+			!is.element(check.char, c("+", "*", ":", "/", "-", "^", "(", "%"))
+		}
+		.variables <- Variables()
+		word <- paste("\\[", gettextRcmdr("factor"), "\\]", sep="")
+		variables <- paste(.variables,
+			ifelse(is.element(.variables, Factors()), paste("[", gettextRcmdr("factor"), "]", sep=""), ""))
+		xBox <- variableListBox(frame, variables, selectmode="multiple", title=gettextRcmdr("Variables (double-click to formula)"))
+		onDoubleClick <- if (!hasLhs){
+				function(){
+					var <- getSelection(xBox)
+					tkselection.clear(xBox$listbox, "0", "end")					
+					if (length(grep(word, var)) == 1) var <- sub(word, "",  var)
+					tkfocus(rhsEntry)
+					rhs <- tclvalue(rhsVariable)
+					rhs.chars <- rev(strsplit(rhs, "")[[1]])
+					check.char <- if (length(rhs.chars) > 0){
+							if ((rhs.chars[1] != " ") || (length(rhs.chars) == 1))
+								rhs.chars[1] else rhs.chars[2]
+						}
+						else ""
+					tclvalue(rhsVariable) <- if (rhs == "" ||
+							is.element(check.char, c("+", "*", ":", "/", "-", "^", "(", "%")))
+							paste(rhs, var, sep="")
+						else paste(rhs, "+", var)
+					tkicursor(rhsEntry, "end")
+					tkxview.moveto(rhsEntry, "1")
+				}
+			}
+			else{
+				function(){
+					var <- getSelection(xBox)
+					which <- tkcurselection(xBox$listbox)
+					tkselection.clear(xBox$listbox, "0", "end")
+					if (length(grep(word, var)) == 1) var <- sub(word, "",  var)
+					lhs <- tclvalue(lhsVariable)
+					if (lhs == "") tclvalue(lhsVariable) <- var
+					else {
+						tkfocus(rhsEntry)
+						rhs <- tclvalue(rhsVariable)
+						rhs.chars <- rev(strsplit(rhs, "")[[1]])
+						check.char <- if (length(rhs.chars) > 0){
+								if ((rhs.chars[1] != " ") || (length(rhs.chars) == 1))
+									rhs.chars[1] else rhs.chars[2]
+							}
+							else ""
+						tclvalue(rhsVariable) <- if (rhs == "" ||
+								is.element(check.char, c("+", "*", ":", "/", "-", "^", "(", "%")))
+								paste(rhs, var, sep="")
+							else paste(rhs, "+", var)
+					}
+					tkicursor(rhsEntry, "end")
+					tkxview.moveto(rhsEntry, "1")
+				}
+			}
+		tkbind(xBox$listbox, "<Double-ButtonPress-1>", onDoubleClick)
+		onPlus <- function(){
+			rhs <- tclvalue(rhsVariable)
+			var <- getSelection(xBox)
+			tkselection.clear(xBox$listbox, "0", "end")										
+			if ((check <- !checkAddOperator(rhs)) && length(var) == 0) return()
+			if (length(var) > 1){
+				if (length(grep(word, var)) > 0) var <- sub(word, "",  var)
+				if (length(var) > 1) var <- paste(var, collapse=" + ")
+			}
+			tclvalue(rhsVariable) <- paste(rhs, if (!check) " + ", var, sep="")
+			tkicursor(rhsEntry, "end")
+			tkxview.moveto(rhsEntry, "1")
+		}
+		onTimes <- function(){
+			rhs <- tclvalue(rhsVariable)
+			var <- getSelection(xBox)
+			tkselection.clear(xBox$listbox, "0", "end")						
+			if ((check <- !checkAddOperator(rhs)) && length(var) == 0) return()
+			if (length(var) > 1){
+				if (length(grep(word, var)) > 0) var <- sub(word, "",  var)
+				var <- trim.blanks(var)
+				if (length(var) > 1) var <- paste(var, collapse="*")
+				tclvalue(rhsVariable) <- paste(rhs, if (!check) " + ", var, sep="")
+			}
+			else tclvalue(rhsVariable) <- paste(rhs, if (!check) "*", sep="")
+			tkicursor(rhsEntry, "end")
+			tkxview.moveto(rhsEntry, "1")
+		}
+		onColon <- function(){
+			rhs <- tclvalue(rhsVariable)
+			var <- getSelection(xBox)
+			tkselection.clear(xBox$listbox, "0", "end")						
+			if ((check <- !checkAddOperator(rhs)) && length(var) == 0) return()
+			if (length(var) > 1){
+				if (length(grep(word, var)) > 0) var <- sub(word, "",  var)
+				var <- trim.blanks(var)
+				if (length(var) > 1) var <- paste(var, collapse=":")
+				tclvalue(rhsVariable) <- paste(rhs, if (!check) " + ", var, sep="")
+			}
+			else tclvalue(rhsVariable) <- paste(rhs, if (!check) ":", sep="")
+			tkicursor(rhsEntry, "end")
+			tkxview.moveto(rhsEntry, "1")
+		}
+		onSlash <- function(){
+			rhs <- tclvalue(rhsVariable)
+			if (!checkAddOperator(rhs)) return()
+			tclvalue(rhsVariable) <- paste(rhs, "/",  sep="")
+			tkicursor(rhsEntry, "end")
+			tkxview.moveto(rhsEntry, "1")
+		}
+		onIn <- function(){
+			rhs <- tclvalue(rhsVariable)
+			if (!checkAddOperator(rhs)) return()
+			tclvalue(rhsVariable) <- paste(rhs, "%in% ")
+			tkicursor(rhsEntry, "end")
+			tkxview.moveto(rhsEntry, "1")
+		}
+		onMinus <- function(){
+			rhs <- tclvalue(rhsVariable)
+			if (!checkAddOperator(rhs)) return()
+			tclvalue(rhsVariable) <- paste(rhs, "- ")
+			tkicursor(rhsEntry, "end")
+			tkxview.moveto(rhsEntry, "1")
+		}
+		onPower <- function(){
+			rhs <- tclvalue(rhsVariable)
+			if (!checkAddOperator(rhs)) return()
+			tclvalue(rhsVariable) <- paste(rhs, "^", sep="")
+			tkicursor(rhsEntry, "end")
+			tkxview.moveto(rhsEntry, "1")
+		}
+		onLeftParen <- function(){
+			tkfocus(rhsEntry)
+			rhs <- tclvalue(rhsVariable)
+			tclvalue(rhsVariable) <- paste(rhs, "(", sep="")
+			tkicursor(rhsEntry, "end")
+			tkxview.moveto(rhsEntry, "1")
+		}
+		onRightParen <- function(){
+			rhs <- tclvalue(rhsVariable)
+			if (!checkAddOperator(rhs)) return()
+			tclvalue(rhsVariable) <- paste(rhs, ")", sep="")
+			tkicursor(rhsEntry, "end")
+			tkxview.moveto(rhsEntry, "1")
+		}
+		outerOperatorsFrame <- tkframe(frame)
+		operatorsFrame <- tkframe(outerOperatorsFrame)
+		plusButton <- buttonRcmdr(operatorsFrame, text="+", width="3", command=onPlus)
+		timesButton <- buttonRcmdr(operatorsFrame, text="*", width="3", command=onTimes)
+		colonButton <- buttonRcmdr(operatorsFrame, text=":", width="3", command=onColon)
+		slashButton <- buttonRcmdr(operatorsFrame, text="/", width="3", command=onSlash)
+		inButton <- buttonRcmdr(operatorsFrame, text="%in%", width="5", command=onIn)
+		minusButton <- buttonRcmdr(operatorsFrame, text="-", width="3", command=onMinus)
+		powerButton <- buttonRcmdr(operatorsFrame, text="^", width="3", command=onPower)
+		leftParenButton <- buttonRcmdr(operatorsFrame, text="(", width="3", command=onLeftParen)
+		rightParenButton <- buttonRcmdr(operatorsFrame, text=")", width="3", command=onRightParen)
+		
+		tkgrid(plusButton, timesButton, colonButton, slashButton, inButton, minusButton,
+			powerButton, leftParenButton, rightParenButton, sticky="w")
+		formulaFrame <- tkframe(frame)
+		if (hasLhs){
+			tkgrid(labelRcmdr(outerOperatorsFrame, text=gettextRcmdr("Model Formula:     "), fg="blue"), operatorsFrame)
+			lhsVariable <- if (currentModel) tclVar(currentFields$lhs) else tclVar("")
+			rhsVariable <- if (currentModel) tclVar(currentFields$rhs) else tclVar("")
+			rhsEntry <- ttkentry(formulaFrame, width="50", textvariable=rhsVariable)
+			rhsXscroll <- ttkscrollbar(formulaFrame,
+				orient="horizontal", command=function(...) tkxview(rhsEntry, ...))
+			tkconfigure(rhsEntry, xscrollcommand=function(...) tkset(rhsXscroll, ...))
+			lhsEntry <- ttkentry(formulaFrame, width="10", textvariable=lhsVariable)
+			lhsScroll <- ttkscrollbar(formulaFrame,
+				orient="horizontal", command=function(...) tkxview(lhsEntry, ...))
+			tkconfigure(lhsEntry, xscrollcommand=function(...) tkset(lhsScroll, ...))
+			tkgrid(lhsEntry, labelRcmdr(formulaFrame, text=" ~    "), rhsEntry, sticky="w")
+			tkgrid(lhsScroll, labelRcmdr(formulaFrame, text=""), rhsXscroll, sticky="w")
+			tkgrid.configure(lhsScroll, sticky="ew")
+		}
+		else{
+			rhsVariable <- if (currentModel) tclVar(currentFields$rhs) else tclVar("")
+			rhsEntry <- ttkentry(formulaFrame, width="50", textvariable=rhsVariable)
+			rhsXscroll <- ttkscrollbar(formulaFrame,
+				orient="horizontal", command=function(...) tkxview(rhs, ...))
+			tkconfigure(rhsEntry, xscrollcommand=function(...) tkset(rhsXscroll, ...))
+			tkgrid(labelRcmdr(formulaFrame, text="   ~ "), rhsEntry, sticky="w")
+			tkgrid(labelRcmdr(formulaFrame, text=""), rhsXscroll, sticky="w")
+		}
+		tkgrid.configure(rhsXscroll, sticky="ew")
+	})
 
 exists.method <- function(generic, object, default=TRUE, strict=FALSE){
     classes <- class(object)
@@ -1797,7 +2030,10 @@ MessagesWindow <- function() getRcmdr("messagesWindow")
 
 activeDataSetP <- function() !is.null(ActiveDataSet())
 
-dataSetsP <- function() !is.null(listDataSets())
+dataSetsP <- function(n=1){
+	datasets <- listDataSets()
+	(!is.null(datasets)) && length(datasets) >= n
+}
 
 numericP <- function(n=1) activeDataSetP() && length(listNumeric()) >= n
 
@@ -1815,6 +2051,8 @@ lmP <- function() activeModelP() && any(class(get(ActiveModel()))[1] == c('lm',
 #glmP <- function() activeModelP() && eval(parse(text=paste("class(", ActiveModel(), ")[1] == 'glm'")))
 glmP <- function() activeModelP() && class(get(ActiveModel()))[1] == 'glm'
 
+aicP <- function() activeModelP() && exists.method("extractAIC", get(ActiveModel()))
+
 polrP <- function() activeModelP() && class(get(ActiveModel()))[1] == 'polr'
 
 multinomP <- function() activeModelP() && class(get(ActiveModel()))[1] == 'multinom'
@@ -2165,4 +2403,55 @@ sortVarNames <- function(x){
 	}
 	ord <- do.call("order", sort.helper(x))
 	x[ord]
-}  
+}
+
+# to load packages
+
+Library <- function(package, pos=4){
+	loaded <- search()
+	loaded <- loaded[grep("^package:", loaded)]
+	loaded <- sub("^package:", "", loaded)
+	if (!getRcmdr("suppress.X11.warnings")){
+		messages.connection <- file(open="w+")
+		sink(messages.connection, type="message")
+		on.exit({
+				sink(type="message")
+				close(messages.connection)
+			})
+	}
+	if (!(package %in% loaded)){
+		command <- paste("library(", package, ", pos=", pos, ")", sep="")
+		logger(command)
+		result <- try(eval(parse(text=command), envir=.GlobalEnv), silent=TRUE)
+		if (class(result)[1] ==  "try-error"){
+			Message(message=paste(strsplit(result, ":")[[1]][2]), type="error")
+			tkfocus(CommanderWindow())
+			return("error")
+		}
+		return(package)
+	}
+	else return(invisible(NULL))
+}
+
+# to merge data frames by rows
+
+mergeRows <- function(X, Y, common.only=FALSE, ...){
+	UseMethod("mergeRows")
+}
+
+mergeRows.data.frame <- function(X, Y, common.only=FALSE, ...){
+	cols1 <- names(X)
+	cols2 <- names(Y)
+	if (common.only){
+		common <- intersect(cols1, cols2)
+		rbind(X[, common], Y[, common])
+	}
+	else {
+		all <- union(cols1, cols2)
+		miss1 <- setdiff(all, cols1)
+		miss2 <- setdiff(all, cols2)
+		X[, miss1] <- NA
+		Y[, miss2] <- NA
+		rbind(X, Y)
+	}
+}
